; Assembly Header File
; Created 7/30/2002, 1:13:45 AM

;graphlib@0015:
show_dialog:

;----------------------------------------------------------------------------
; show_dialog()
;
; Function: Displays a dialog box.
;
; input: A6=pointer to dialog struct
; output: nothing
;
;Dialog struct:
;0(a6).w : x1 (top-left)
;2(a6).w : y1 (top-left)
;4(a6).w : x2 (down-right)
;6(a6).w : y2 (down-right)
;8(a6).w : x of the string relative to x1
;10(a6).w :y of the string relative to y1
;12(a6).l : adress of the string
;
;example of dialog structure
;
;dial	dc.w	10,15,50,40,10,5
;	dc.l	str
;
;str	dc.b	"HELLO !",0
;----------------------------------------------------------------------------

	movem.l 	d0-d7/a0-a6,-(a7)	;save registers
	move.w		(a6)+,d0		;x1 in d0
	move.w		(a6)+,d1		;y1 in d1
	move.w		(a6)+,d4		;x2 in d4
	move.w		(a6)+,d5		;y2 in d5
	sub.w		d0,d4			;d4=width
	sub.w		d1,d5			;d5=height

	lea		dialog(pc),a3
	movem.w	d0/d1/d4/d5,(a3)	;store d0,d1,d4,d5 to dialog
	bsr		clear_dialog		;clear the box
	addq.w		#1,d0			;x1+1
	addq.w		#1,d1			;y1+1
	subq.w		#2,d4			;width-2
	subq.w		#2,d5			;height-2
	bsr		frame			;draw a frame 
	addq.w		#1,d0			;x1+2
	addq.w		#1,d1			;y1+2
	subq.w		#2,d4			;width-4
	subq.w		#2,d5			;height-4
	bsr		frame			; a smaller frame
	addq.w		#2,d0			;x1+4
	addq.w		#2,d1			;y1+4
	subq.w		#4,d4			;width-8
	subq.w		#4,d5			;height-8
	bsr		frame			;a smaller frame
	move.w		#2,-(a7)
;	jsr		doorsos::FontSetSys
	ROM_CALL	FontSetSys
	addq.l		#2,a7			;new font:2
	move.b		d0,d5			;d5 = old_font
	movem.w	(a3),d0/d1		;restore d0-d1 from dialog
	move.w		#4,-(a7)
	move.w		(a6)+,d2		;d2=x of text rel to x1
	move.w		(a6)+,d3		;d3=y of text rel to y1
	move.l		(a6),-(a7)		;address of the message in stack
	add.w		d0,d2			;d2=x+x1:
	add.w		d1,d3;d3=y+y1
	move.w		d3,-(a7)
	move.w		d2,-(a7)		;real coordinates of the message in stack
;	jsr		doorsos::DrawStrXY	;disp the message
	ROM_CALL	DrawStrXY
	lea		10(a7),a7
	move.b		d5,-(a7)
	;jsr		doorsos::FontSetSys	;restore the old-font
	ROM_CALL	FontSetSys
	addq.l		#2,a7

	movem.l 	(a7)+,d0-d7/a0-a6	;restore the registers
	rts					;quit func

;graphlib@0016:
clear_dialog:

;----------------------------------------------------------------------------
; clear_dialog()
;
; Function: Erases the last dialog box drawn by show_dialog(). The area
;           previously under the dialog will need to be redrawn.
;
; input:  nothing
; output: nothing
;----------------------------------------------------------------------------


	movem.l	d0-d7/a0-a6,-(a7)	;save registers
	lea		dialog(pc),a3
	movem.w	(a3),d0/d1/d2/d3	;restore with a3 the four parameters of the box fo fill
	moveq.w	#1,d4			;color: white
	bsr		fill			;clear the box
	movem.l	(a7)+,d0-d7/a0-a6	;restore the old registers
	rts

fill:
;--------------------------------------------------------------
;fill(x,y,width,height,color)
;
;   Fills the rectangle delimited by x, y, width and lenght with 
;  the specified color
;
;Input:d0.w = x
;	d1.w = y
;	d2.w = width
;	d3.w = height
;	d4.w = color:	0 -> Video Invert
;			1 -> White
;			2 -> Black
;
;Output: nothing
;	NO REGISTERS DESTROYED
;--------------------------------------------------------------
	movem.l	d0-d7/a0-a6,-(a7)	; save registers
	exg.w		d4,d3			;  d3 in d4 and d4 in d3 because of horiz: color in d3
	add.w		d0,d2			;  width + x1=x2
\line	bsr		horiz			;  disp the line x1,y1,x2,y1
	addq.w		#1,d1			;+1 to y1
	dbra		d4,\line		;make a loop d4 (height) time
	movem.l	(a7)+,d0-d7/a0-a6	;restore registers
 	rts

;--------------------------------------------------------------
;frame(x,y,width,height)
;
;  Draws a frame at (x,y)
;
;Input: d0.w = x
;	d1.w = y
;	d4.w = width
;	d5.w = height
;	NO REGISTERS DESTROYED
;--------------------------------------------------------------

frame:
	movem.l		d0-d7/a0-a6,-(a7)
	moveq.w	#2,d3			; color black for horiz and vert
	move.w		d0,d2
	add.w		d4,d2 			; d2 = x1 + Width
	bsr		horiz			; upper line horiz
	move.w		d1,d2			;d2:y1
	add.w		d5,d1			;d1:y1+height
	bsr		vert			;draw left vert line
	move.w		d0,d2
	addx.w		d4,d0			; d2 = x1 + Width
	bsr		horiz			;draw horiz bottom line
	move.w		d1,d2			;d2=y1+height
	subx.w		d5,d1			;d1=y1
	bsr		vert			;(d0=x1+width)right line
	movem.l		(a7)+,d0-d7/a0-a6	; restore registers
	rts

vert:
;--------------------------------------------------------------
;vert(x,y1,y2)
;
;   Draws a vertical line
;
;Input:	d0.w = x
;	d1.w = y1
;	d2.w = y2
;
;Output: nothing
;	NO REGISTERS DESTROYED
;--------------------------------------------------------------

	movem.l		d0-d7/a0-a6,-(a7)	;save registers
	cmp.w		d1,d2
	bge		\coolvert			; if d2 greater than d1 go to coolvert 
	exg.w		d1,d2			; if d1 greater
\coolvert bsr		prepare			; go to prepare
	sub.w		d5,d2			;d2= height of the line
\bitset bset		d4,(a1) 			;set the bit d4 of the byte at a1
	lea		30(a1),a1		; a1:address of the byte a line after: a line=30 bytes
	dbra.w		d2,\bitset
	movem.l		(a7)+,d0-d7/a0-a6	;restore registers
	rts

horiz:
;--------------------------------------------------------------
;horiz(x1,y,x2,color)
;
;   Draws an horizontal line in the specified color
;
;Input:	d0.w = x1
;	d1.w = y
;	d2.w = x2
;	d3.w = color  	0 -> Video Invert
;			1 -> White
;			2 -> Black
;
;Output: nothing
;	NO REGISTERS DESTROYED
;--------------------------------------------------------------

	movem.l		d0-d7/a0-a6,-(a7)
	cmp.w		d0,d2
	bge		\coolhr			; if d2 greater than d1go to coolhr
	exg.w		d0,d2			; if d1 greater
\coolhr
	bsr		prepare
	sub.w		d0,d2 			;d2:width of the line
	addq.w		#1,d2
\set	bchg		d4,(a1)			 ;inverse the bit d4 of the byte at a1
	tst.w		d3
	beq		\pixdone			; if color= inverse:next pixel
	bclr		d4,(a1)			;clear the bit d4 of the byte at a1
	cmp.w		#1,d3;
	beq		\pixdone			;if color= white:nextpixel
	bset		d4,(a1)			;else: sets the bit
\pixdone subq.w		#1,d2
	tst.b		d4			; if it is the last bit:76543210
	beq		\bytes
	subq.w		#1,d4			;else: next bit
\test	tst.w		d2			;d2=number of pixels to plot
	bne		\set
	movem.l		(a7)+,d0-d7/a0-a6
	rts
\bytes
 	addq.l		#1,a1			;a1: address of the next byte
	cmp.w		#8,d2			; if d2 is less or equal than 7, it remains one byte:go to finb
	blt		\finb			;else plot the whole byte
	not.b		(a1)			;inverse 
	tst.w		d3			;if color= inverse:OK
	beq		\lndone
	clr.b		(a1);
	cmp.w		#1,d3			;if color=white:OK
	beq		\lndone
	not.b		(a1)		
\lndone subq.w	#7,d2
	dbra		d2,\bytes		;d2-8
\finb	moveq.b	#7,d4				; d4= bit that correspond to the pixel in a byte:7:first bit
	bra		\test
prepare:
	move.w		d0,d4
	move.w		d1,d5
;	bsr		loadscreen		; a1:right address of the screen
	;lea		$4c00,a1
	move.l __D_plane,a1
	mulu.w		#30,d1			; a horiz line:240=30*8 pixels: 30 bytes
	add.l		d1,a1			;a1: address of the start of line d1
	lsr.w		#3,d4; d4=width/8  
	add.w		d4,a1			; a1:address of the bytes which contains the pixel
	move.w		d0,d4       		;    for example:          d0:00010101
	not.b		d4			;                                 d4:11101010
	and.b		#7,d4			;          #7:00000111  d4:00000010
						; d4= bit that correspond to the pixel in a byte:
	rts
	
	